-- A WerwacK s script for 3ds max R8.x

-- wkParticles2Spline
-- Version 2.1
-- 24.10.2006


-- Contact and feedback: werwack@yahoo.com
-- Visit www.werwackscripts.com for free cool scripts ;)

/*
-- Purpose:
  ---------

-- This script allows you to create splines from particles, what is useful for particles coming from
-- RealFlow. The splines will have the number of knots you specify, each knot will follow the animation of 
-- a particle. Knots of the splines (in the lines, in fact, but each line created is constituted by a lonely
-- spline) are animated frame by frame, with, by default, a TCB controller.


-- Notes:
  -------
  
--		 - If you are not working with a RealFlow Particle System, be careful to run the script after the emission start
-- 		   to avoid troubleshootings, indeed particles system methods don't work before emission start.
--		 - RealFlow particles are not stable at first frame(s), thus sometimes the generated splines don't
--		   match with the particles at these frames. This bug will be over when importing rightly RealFlow particles is possible.
--		 - tcb controllers are automatically applied for splines rotation and for the vertices position.
--

-- wip 	 - spline direction is automatically inverted when parent is used
--		 - BE SURE TO SET THE START FRAME WITH THE SAME INDEX THAN THE FRAME INDEX USED IN REALFLOW


-- Known troubles:
   ---------------  

	- WARNING: first key, when parent used, is not reliable!!!
	- particle indices may still be displayed even if the check button to see them is off
	- waring: when using multiple events with particle flow, particle indices are modified every new event
	
-- Todo:
   -----
	- advanced arrays for particle indices: tres facile a ajouter
	- mettre des options pour rendre les plines renderable
	- pouvoir prendre des plines existantes

-- Version history:
  -----------------

-- Version 2.1
-- relased 24.10.2006 : - a big fixed: crash is now avoided when particles are not alive at the given start time

-- Version 2.0
-- relased 10.03.2006 : - convertion for 3DSMax 8.x
						- code optimized (faster, cleaner)
						- particle indices can be displayed in the viewport
						- particle flow emitters are now supported

-- Version 1.21
-- relased 13.06.2003 : - convertion for 3DSMax 5.x

-- Version 1.2
-- relased 13.07.2001 : -the splines pivot is now on the first knot of the spline, and its x axis is on the line made
--						by the first and the second knot.
--						-a check button allows the user to choose between free splines or splines linked to a parent.
--						-problem with multi loops has been solved for phi (rotation around the z axis), but not for teta (rotation
--						around the y axis). The spline may then have a strange behavior between 2 keys. It is advised to create a 
--						key for each frame.

*/

macroScript WkParticles2Spline category:"WerwacK s Tools" tooltip:"Particles2Spline V2.1" Icon:#("WkParticles2SplineIcons",1)
(

--global in the macro :
global WkP2SRollAbout, WkP2SSettingsRoll, WkPart2SplinWind

--local in the macro :
global	gmArParticleIndicesForKnots, \
		gmVerticesNum=1, line_type=#curve, \
		gmParticleSystem = undefined, \
		gmParentObj = undefined,	\		--parent for the splines
		gmValidNumParts = undefined, \		-- number of particles needed by the user, or number of particles in the system if there are not enougth particles to respect the settings
		gmUserCurrentTime = undefined

local GM_PICKEDOBJBUTWIDTH = 60   



------------------------------------------------------------------------------------------
-- returns true if rObj is a particle system or a Particle Flow system, false otherwise
-- WARNING: PF particle systems do not have the same properties than standard particle systems
fn IsParticleSystem rObj =
(
	if (classof rObj) == PF_Source do return true
	
	local lNumparts = undefined
	
	try( lNumparts = particlecount rObj )
	catch lNumparts = undefined
							
	if (undefined != lNumparts) then
		 return true
	else return false
)


-- filter for particle systems (not filtering from the class because too restrictive)
fn PartSysFilter rObj =
(
	if (classof rObj) == PF_Source do return true
	
	local lNumparts = undefined
	
	try( lNumparts = particlecount rObj )
	catch lNumparts = undefined
							
	if (undefined != lNumparts) then
		 return true
	else return false
)

fn ParentFilter rObj =
(
	return true
)

------------------------------------------------------------------------------------------
-- the function "particlecount" gives a wrong particle count, coming from settings inside RealFlow;
-- the particle count is recalculated here
fn FindRealParticleNumber =
(
	if (classof gmParticleSystem) == PF_Source do
		return gmParticleSystem.numParticles()

	ind_particle=1
	
--	while ((particlepos gmParticleSystem ind_particle)!=undefined) do
--		ind_particle+=1
--	ind_particle = ind_particle - 1
	
	ind_particle = particleCount gmParticleSystem
	
	format "\nFindRealParticleNumber - ind_particle: %\n" ind_particle
	return ind_particle
)


------------------------------------------------------------------------------------------
-- fills the array of arrays gmArParticleIndicesForKnots
-- each sub array contains the indices of the particles of the corresponding splines
-- to do: allow user to fill the array himself for each spline
fn FillParticleIndicesForKnots =
(
--	format "FillParticleIndicesForKnots\n"
	
	gmArParticleIndicesForKnots = #()			-- each member of the array contains an array with the particle indices to use for the corresponding spline (ordered)
	local lSplinesNum = WkP2SSettingsRoll.SplineNumberSpin.value
	local lVerticesNum = WkP2SSettingsRoll.num_vertices_spin.value
	local lbInvertSplineDir = WkP2SSettingsRoll.invertsplinedirchkbx.checked
	local lbUsecumulativepartind = WkP2SSettingsRoll.usecumulativepartindchkbx.checked
	
	startind = if lbInvertSplineDir then lVerticesNum else 1
	endind = if lbInvertSplineDir then 1 else lVerticesNum
	stepind = if lbInvertSplineDir then -1 else 1
	
	
	if lbUsecumulativepartind then
		for i=1 to lSplinesNum do
		(
			gmArParticleIndicesForKnots[i] = #()
	
			for j=startind to endind by stepind do
				if gmValidNumParts >= ((i - 1) * lVerticesNum + j) do
					append gmArParticleIndicesForKnots[i] ((i - 1) * lVerticesNum + j)
		)
	else
	(
		for i=1 to lSplinesNum do
		(
			gmArParticleIndicesForKnots[i] = #()
		)

		for i=1 to lSplinesNum do
		(
			for j=startind to endind by stepind do
				if gmValidNumParts >= ((j - 1) * lSplinesNum + i) do
					append gmArParticleIndicesForKnots[i] ((j - 1) * lSplinesNum + i)
		)
	)
		
	--wkip
/*	if not lbInvertSplineDir then
		gmArParticleIndicesForKnots[1] = #(1,2,3,4)
	else gmArParticleIndicesForKnots[1] = #(4,3,2,1)
*/
	
	format "gmArParticleIndicesForKnots:\n"
	for i=1 to lSplinesNum do
		format "   gmArParticleIndicesForKnots[%]: %\n" i gmArParticleIndicesForKnots[i]	
)


------------------------------------------------------------------------------------------
-- creation of free splines, each particle corresponding to the knot of a spline
-- use rArParticleIndicesForKnots
fn CreateAndAnimateSplines =
(
	format "CreateAndAnimateSplines\n"
	
	local lUsedParticlesNum = 1
	local lStartTimeVal = WkP2SSettingsRoll.StartFrameSpin.value
	local lEndTimeVal = WkP2SSettingsRoll.EndFrameSpin.value
	local lIntervTime = WkP2SSettingsRoll.IntervFrameSpin.value
	local lStepsNum = WkP2SSettingsRoll.SplineStepsSpin.value
	local lbSplinefollowsfirstpart = WkP2SSettingsRoll.splinefollowsfirstpartchkbx.checked
	local lbInvertSplineDir = WkP2SSettingsRoll.invertsplinedirchkbx.checked
	
	-- Progress Bar Start
	ProgressStart ("Creating Splines... Wait")
	
	--create splines (with at least 2 knots)
	local lArSplines = #()
	for lSplineInd = 1 to gmArParticleIndicesForKnots.count do
	if 1 < (gmArParticleIndicesForKnots[lSplineInd]).count do
	(
		
	-- create spline
		spline = line steps:lStepsNum name:(uniquename ("WkLine_" + gmParticleSystem.name + "_" + lSplineInd as string)) \
					position:[0,0,0]
		if WkP2SSettingsRoll.parent_chkbx.checked do
		(
			spline.parent = gmParentObj
			in coordsys #parent spline.position = [0,0,0]
		)

		spline.vertexTicks = off
		spline.wirecolor = (color 28 89 177)
		spline.render_thickness = 1.5
		spline.render_displayRenderMesh = false
		spline.render_renderable = true
		spline.render_useViewportSettings = true
		spline.adaptive = true
						
		addnewspline spline
			
		-- add vertices
		local lVertexInd = 1
		for i=1 to gmArParticleIndicesForKnots[lSplineInd].count do
		(
			--wkip line type a remplir 
			--wkip we assume part count is ok in the knots array
			addknot spline 1 #smooth line_type [0,0,0]
		)
			
		updateshape spline

	-- animate vertices
		-- animate vertices only (not in and out tangents)
		for i=1 to (numknots spline) do		-- and not numverts to prevent crash if the last spline has not numverts knots
		(
			animatevertex spline (3*i - 1)
--			spline[4][8][3*(numknots spline - i) + 2].controller = tcb_point3()		-- assignation of a tcb position controller for every vertex of the splines
		)
		updateshape spline

		for i=1 to (numknots spline) do		-- and not numverts to prevent crash if the last spline has not numverts knots
		(
--			animatevertex spline (3*i - 1)
			spline[4][8][3*i - 1].controller = tcb_point3()		-- assignation of a tcb position controller for every vertex of the splines
		)

		spline.position.controller = tcb_position()		-- assignation of a tcb position controller for the splines position
		spline.rotation.controller = tcb_rotation()		-- assignation of a tcb rotation controller for the splines rotation
		
		append lArSplines spline
	)
	
	ProgressUpdate (15)

	
	-- affect for each specified frame (the interval between frames depends on lIntervTime) particles position to knots
	phi=0
	numloopphi=0

	local lTimeVal = lStartTimeVal
	while lTimeVal <= lEndTimeVal do
	(
		ProgressUpdate (15 + 85.*lTimeVal/lEndTimeVal)
		
		local spline
		at time (lTimeVal as time)
			for lSplineInd=1 to lArSplines.count do
			(
				spline = lArSplines[lSplineInd]
				
				sliderTime = (lTimeVal as time)

				local firstknotpos, secondknotpos

				if (classof gmParticleSystem) == PF_Source then
					in coordsys #parent
					(
						firstknotpos = gmParticleSystem.getparticleposition (gmArParticleIndicesForKnots[lSplineInd][1])
						secondknotpos = gmParticleSystem.getparticleposition (gmArParticleIndicesForKnots[lSplineInd][2])
					)
				else in coordsys #parent
				(
					firstknotpos = particlepos gmParticleSystem (gmArParticleIndicesForKnots[lSplineInd][1])
					secondknotpos = particlepos gmParticleSystem (gmArParticleIndicesForKnots[lSplineInd][2])
				)
				
				local lbIsParticlesValid = true		
				if undefined == firstknotpos do
				(
					format "-- Time: %, Spline Index: %: First particle is not alive\n" lTimeVal lSplineInd firstknotpos
					lbIsParticlesValid  = false
				)
				
				if undefined == secondknotpos do
				(
					format "-- Time: %, Spline Index: %: Second particle is not alive\n" lTimeVal lSplineInd secondknotpos
					lbIsParticlesValid  = false
				)
				
				if lbIsParticlesValid  do
				(
					--rotation between first and second knot, for axis alignment
					hypk1k2=distance firstknotpos secondknotpos
					
					--teta e [-90,90]
					if hypk1k2 > 0
					then teta=asin((secondknotpos.z - firstknotpos.z)/hypk1k2)
					else teta=0
				
					--phi
					prevphi=phi --prevphi e [-180,180]
					hypprojk1projk2=distance [firstknotpos.x, firstknotpos.y, 0] [secondknotpos.x, secondknotpos.y, 0]
					if (secondknotpos.y - firstknotpos.y)==0 --knots are on the same plane xoz
					then if (secondknotpos.x - firstknotpos.x) >= 0 --knots are on the same origin
						then phi=0
						else if ( prevphi>0 )
							then phi=180
							else phi=-180
					else
					(
						if (secondknotpos.x - firstknotpos.x)==0
						then if (secondknotpos.y > firstknotpos.y)
							then phi=90
							else phi=-90
						else
						(
							phi=acos((secondknotpos.x - firstknotpos.x)/hypprojk1projk2)
							if (secondknotpos.y < firstknotpos.y) do phi=-phi
							--solves the pb with loops
							if (abs(phi-prevphi)>=300 ) do
								if phi<prevphi --trigonometric rotation
								then numloopphi+=1
								else numloopphi-=1
						)
					)

					rotteta=quat -teta [0,1,0]
					rotphi=quat (phi + 360*numloopphi) [0,0,1]


					local lFirstPartInd = 1
					local lLastPartInd = (gmArParticleIndicesForKnots[lSplineInd]).count
				
					if lbsplinefollowsfirstpart do
					(
						lFirstPartInd = 2
					--anim the spline pos and rot to the emitter
						--rot
						addnewkey spline.rotation.controller (lTimeVal as time)
						k_ind=getkeyindex spline.rotation.controller (lTimeVal as time)

						if WkP2SSettingsRoll.parent_chkbx.checked then
							in coordsys #parent
								spline.rotation.controller.keys[k_ind].value = ( rotphi * rotteta * (inverse gmParentObj.transform) * (inverse spline.rotation)) as angleaxis
						else in coordsys #parent
								spline.rotation.controller.keys[k_ind].value = ( rotphi * rotteta * (inverse spline.rotation) ) as angleaxis
						

						--pos
						addnewkey spline.position.controller (lTimeVal as time)
						k_ind=getkeyindex spline.position.controller (lTimeVal as time)
						
						if (classof gmParticleSystem) == PF_Source then
						(
							in coordsys gmParticleSystem
								firstknotpos = gmParticleSystem.getparticleposition (gmArParticleIndicesForKnots[lSplineInd][1])
							
							if WkP2SSettingsRoll.parent_chkbx.checked then
								in coordsys gmParticleSystem
									spline.position.controller.keys[k_ind].value = firstknotpos * (inverse gmParentObj.transform)
							else in coordsys gmParticleSystem
									spline.position.controller.keys[k_ind].value = firstknotpos
						)
						else in coordsys gmParticleSystem
							(
								firstknotpos = particlepos gmParticleSystem (gmArParticleIndicesForKnots[lSplineInd][1])
								if WkP2SSettingsRoll.parent_chkbx.checked then
									in coordsys #parent
										spline.position.controller.keys[k_ind].value = firstknotpos * gmParticleSystem.transform * (inverse gmParentObj.transform)
								else in coordsys #world
										spline.position.controller.keys[k_ind].value = firstknotpos * gmParticleSystem.transform
							)
					)

				
					--anim knots at the given frame
					for i=lFirstPartInd to lLastPartInd do
					(
					--	i = gmArParticleIndicesForKnots[lSplineInd]
				--		if( undefined == spline[4][8][3*(numknots spline - i) + 2].controller )
				--			spline[4][8][3*(numknots spline - i) + 2].controller = 
							
						-- the vertex tracks are in the 4th subanim (Object__Line), in the 1st sub-subanim (master) and entrelaced with in:out tangent tracks
						addnewkey spline[4][8][3*i - 1].controller (lTimeVal as time)
						k_ind=getkeyindex spline[4][8][3*i - 1].controller (lTimeVal as time)
						
						if (classof gmParticleSystem) == PF_Source then
						in coordsys #parent
						(
						--	firstknotpos = gmParticleSystem.getparticleposition ((gmArParticleIndicesForKnots[lSplineInd][1])
						--	format " - firstknotpos: %\n" firstknotpos
							partmat = gmParticleSystem.getparticleposition (gmArParticleIndicesForKnots[lSplineInd][i]) -- - firstknotpos )
						--	spline[4][8][3*(numknots spline - i) + 2].keys[k_ind].value = partmat --* (inverse gmParticleSystem.objecttransform)--* rotphi * rotteta --* gmParticleSystem.transform
							if undefined != partmat do
								spline[4][8][3*i - 1].keys[k_ind].value = partmat * (inverse spline.objecttransform)--* rotphi * rotteta --* gmParticleSystem.transform
						)
						else in coordsys #parent
							(
							--	firstknotpos=particlepos gmParticleSystem (gmArParticleIndicesForKnots[lSplineInd][i])
								partmat = particlepos gmParticleSystem (gmArParticleIndicesForKnots[lSplineInd][i])
								if undefined != partmat do
									spline[4][8][3*i - 1].keys[k_ind].value = partmat * (inverse spline.objecttransform)--rotphi * rotteta
							)
					)
				)
			)
				
			lTimeVal+=lIntervTime
		)
	
	ProgressEnd()

	
	-- create a named selection set
	selectionsets[("WkLines_" + gmParticleSystem.name)] = lArSplines
	select lArSplines		
	
	slidertime = gmUserCurrentTime
)
	

	
fn drawPointCoordCallback =
(
	if 8000 <= (maxversion())[1] and undefined != gmParticleSystem and undefined != gmParticleSystem then
	(
		gw.setTransform(Matrix3 1)
		local lColor = [200,0,0]
		local lParticlesNum, lPartPos
		
		if (classof gmParticleSystem) == PF_Source then
		(
			lParticlesNum = gmParticleSystem.numparticles()
			
			for partInd=1 to lParticlesNum do
			--	if "Point" == ((classof obj) as string) do
				(
					lim = gw.getRndLimits()
					append lim #polyEdges
					append lim #illum
					gw.setRndLimits lim
					
					lPartPos = (gmParticleSystem.getparticleposition partInd) + [2,2,2]
			--		wkip doesn t work with max 8!!	
			--		p = gw.hTransPoint lPartPos
			--		gw.hPolyline #(p+[-10,18,8], p+[35,18,8], p+[35,-10,8], p+[-10,-10,8]) true
					p = gw.hTransPoint lPartPos
					gw.hText p (partInd as string) color:lColor
			--		gw.hMarker p #smallHollowBox color:lColor
				)
		)
		else
		(
			lParticlesNum = particleCount gmParticleSystem
			
			for partInd=1 to lParticlesNum do
			--	if "Point" == ((classof obj) as string) do
				(
					lim = gw.getRndLimits()
					append lim #polyEdges
					append lim #illum
					gw.setRndLimits lim
					
					lPartPos = (particlepos gmParticleSystem partInd) + [2,2,2]
			--		wkip doesn t work with max 8!!	
			--		p = gw.hTransPoint lPartPos
			--		gw.hPolyline #(p+[-10,18,8], p+[35,18,8], p+[35,-10,8], p+[-10,-10,8]) true
					p = gw.hTransPoint lPartPos
					gw.hText p (partInd as string) color:lColor
			--		gw.hMarker p #smallHollowBox color:lColor
				)			
		)
	
		gw.enlargeUpdateRect #whole
		gw.updateScreen()
	)
)


fn UpdateGui =
(
	unregisterRedrawViewsCallback drawPointCoordCallback

	if undefined != gmParticleSystem and undefined != gmParticleSystem then
	(
		if (classof gmParticleSystem) == PF_Source then	
			 WkP2SSettingsRoll.numpartsvallab.text = gmParticleSystem.numparticles() as string
		else WkP2SSettingsRoll.numpartsvallab.text = particleCount gmParticleSystem as string
		
		if true == WkP2SSettingsRoll.viewParticleIndchkbut.checked then
			registerRedrawViewsCallback drawPointCoordCallback
		else unregisterRedrawViewsCallback drawPointCoordCallback
	)
	else
	(
		gmParticleSystem = undefined
		WkP2SSettingsRoll.partsysedittxt.text = ""
		WkP2SSettingsRoll.numpartsvallab.text = "-"
		
		WkP2SSettingsRoll.viewParticleIndchkbut.checked = false
		unregisterRedrawViewsCallback drawPointCoordCallback
	)

	forceCompleteRedraw()
)
	


------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
-- Rollout for credits
Rollout WkP2SRollAbout "About..."
(
	label prodlab "A WerwacK prod."
	label verslab "Version 2.1 - Max 8.x"
	label datelab "Released:24.10.2006"
)
	

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
-- Settings
Rollout WkP2SSettingsRoll "Settings"
(
	group "Particle Emitter "
	(
		pickbutton partsyspckbut "Part. Sys." message:"Pick a Particle System" tooltip:"Pick a Particle System" filter:PartSysFilter height:18 width:56 align:#left offset:[-4,0]
		button objfromselbut "$" tooltip:"Get Object From Selection" height:18 width:10 align:#left offset:[GM_PICKEDOBJBUTWIDTH - 8 + 2,-23]
		button selobjbut "S" tooltip:"Select Object" height:18 width:10 align:#left offset:[GM_PICKEDOBJBUTWIDTH - 8 + 12,-23]
		edittext partsysedittxt text:"" width:130 align:#left offset:[74,-23]
		
		label numpartslab "Particles Num. at current time:" align:#left offset:[-4,0]
		label numpartsvallab "-" align:#left offset:[144,-18]
		button numpartsupdatebut "Up" tooltip:"Update" height:16 align:#right offset:[0,-18]
		
		checkbutton viewParticleIndchkbut "Part. Indices" tooltip:"View Particle Indices" height:18 width:70 align:#right offset:[4,0]
	)
	
	on partsyspckbut picked particlesyst do
	(
		gmParticleSystem = particlesyst
		partsysedittxt.text = particlesyst.name
		
		UpdateGui()
	)

	on objfromselbut pressed do
	(
		if 0 == selection.count then
			 messagebox "Please Select a Particle Emitter"
		else if 1 < selection.count then
			 messagebox "Please Select Only 1 Particle Emitter"
		else if PartSysFilter $ do
		(
			gmParticleSystem = $
			partsysedittxt.text = gmParticleSystem.name
			
			UpdateGui()
		)
	)
	
	on selobjbut pressed do
	(
		if undefined != gmParticleSystem do
		(
			if (IsValidNode gmParticleSystem) then
			(
				if gmParticleSystem.isfrozen or gmParticleSystem.ishidden then
					if querybox ("Object is Hidden or Frozen \nMake it selectable?") do
					(
						gmParticleSystem.isfrozen = false
						gmParticleSystem.ishidden = false
						select gmParticleSystem
					)
				else select gmParticleSystem
			)
			else
			(
				messagebox "Deleted or Invalid Object"
				gmParticleSystem = undefined
				partsysedittxt.text = ""
			)
		)
		
		UpdateGui()
	)
	
	on viewParticleIndchkbut changed stat do
	(
		unregisterRedrawViewsCallback drawPointCoordCallback
		if stat do
			if undefined != gmParticleSystem and undefined != gmParticleSystem then
				registerRedrawViewsCallback drawPointCoordCallback
			else viewParticleIndchkbut.checked = false
		forceCompleteRedraw()
	)
	
	on numpartsupdatebut pressed do
		UpdateGui()
		
	group "Parent "
	(
		checkbox parent_chkbx "Link Splines to Object" checked:false align:#left offset:[-4,-2]
		pickbutton parent_pickbut "Parent" message:"Pick a Parent for the Splines" tooltip:"Pick a Parent for the Splines" filter:ParentFilter height:18 width:56 align:#left offset:[-4,-2] enabled:false
		button parentfromselbut "$" tooltip:"Get Object From Selection" height:18 width:10 align:#left offset:[GM_PICKEDOBJBUTWIDTH - 8 + 2,-23] enabled:false
		button selparentbut "S" tooltip:"Select Object" height:18 width:10 align:#left offset:[GM_PICKEDOBJBUTWIDTH - 8 + 12,-23] enabled:false
		edittext parent_txt "" text:"" width:130 align:#left offset:[74,-23]	
	)
	
	on parent_chkbx changed stat do
	(
		parent_pickbut.enabled = stat
		parent_txt.enabled = stat
		parentfromselbut.enabled = stat
		selparentbut.enabled = stat
		
		if not stat then
		(
			parent_txt.text = ""
			gmParentObj = undefined
		)
	)

	on parent_pickbut picked parentobject do
	(
		gmParentObj = parentobject
		parent_txt.text = gmParentObj.name
	)

	on parentfromselbut pressed do
	(
		if 0 == selection.count then
			 messagebox "Please Select an Object"
		else if 1 < selection.count then
			 messagebox "Please Select Only 1 object"
		else if ParentFilter $ do
		(
			gmParentObj = $
			parent_txt.text = gmParentObj.name
		)
	)
	
	on selparentbut pressed do
	(
		if undefined != gmParentObj do
		(
			if (IsValidNode gmParentObj) then
			(
				if gmParentObj.isfrozen or gmParentObj.ishidden then
					if querybox ("Object is Hidden or Frozen \nMake it selectable?") do
					(
						gmParentObj.isfrozen = false
						gmParentObj.ishidden = false
						select gmParentObj
					)
				else select gmParentObj
			)
			else
			(
				messagebox "Deleted or Invalid Object"
				gmParentObj = undefined
				parent_txt.text = ""
			)
		)
	)
		
		
	-- user has to specify the number of splines, the number of vertices and steps per spline
	group "Splines "
	(
		label num_splin_lab "Number of Splines  :" align:#left offset:[-4,-3]
		spinner SplineNumberSpin "" range:[1, 999, 1] type:#integer offset:[20,-18] width:45
		label num_vertices_lab "Vertices per Spline : " align:#left offset:[-4,-3]
		spinner num_vertices_spin "" range:[1, 2000, 5] type:#integer offset:[20,-18] width:45
		label steps_lab "Steps : " align:#left offset:[-4,-3]
		spinner SplineStepsSpin "" range:[1, 20, 6] type:#integer offset:[20,-18] width:45
		
		checkbox usecumulativepartindchkbx "Use Cumulative Particle Indices" checked:false offset:[15,-2] enabled:true

		label type_lab "Type : " align:#left offset:[-4,3]
		radiobuttons type_radbut "" labels:#("Corner","Smooth") default:2 offset:[20,-18]
		
		checkbox invertsplinedirchkbx "Invert Spline Direction" checked:false offset:[15,-2] enabled:true
		checkbox splinefollowsfirstpartchkbx "Align Pivot on First Particle" checked:true offset:[15,-2] enabled:true
	)

	spinner StartFrameSpin "From : " range:[-2*(animationrange.end-animationrange.start), animationrange.end, animationrange.start] type:#integer align:#left offset:[4,0] width:65
	spinner EndFrameSpin "To : " range:[StartFrameSpin.value, 2*(animationrange.end-animationrange.start), animationrange.end] type:#integer width:65 offset:[25,-21]

	spinner IntervFrameSpin "Every  " range:[1,100,1] type:#integer width:65 align:#left offset:[4,0]
	label IntervFrameLab "Frame" align:#left offset:[110,-20]
	
	button apply_but "Apply" width:163 align:#center offset:[0,3]

	on num_vertices_spin changed val do
	(
		gmVerticesNum=val
	)
	
	on type_radbut changed stat do
	(
		if stat==1
		then line_type=#line
		else line_type=#curve
	)
		
	on StartFrameSpin changed val do
	(
		EndFrameSpin.range=[val + 1, 2*(animationrange.end-animationrange.start), EndFrameSpin.value]
	)

	on EndFrameSpin changed val do
	(
		StartFrameSpin.range=[-2*(animationrange.end-animationrange.start), val - 1, StartFrameSpin.value]
	)

	on apply_but pressed do
	(
		gmVerticesNum = num_vertices_spin.value
		gmValidNumParts = SplineNumberSpin.value * gmVerticesNum
		gmRealNumParts = FindRealParticleNumber()
		
		--SplineNumberSpin
		local lbContinue = true
		
		gmUserCurrentTime = slidertime
		slidertime = WkP2SSettingsRoll.StartFrameSpin.value
		
		if undefined == gmParticleSystem then
			messagebox "Please, Pick a Particle System First" title:"No Particle System"
		else if not (isValidNode gmParticleSystem) then
		(
			gmParticleSystem = undefined
			partsysedittxt.text = ""
			messagebox "Please, Pick a Particle System First" title:"No Particle System"
		)
		else if (PF_Source == (classof gmParticleSystem) and gmValidNumParts > gmParticleSystem.numParticlesGenerated()) \
			or (PF_Source != (classof gmParticleSystem) and gmRealNumParts < gmValidNumParts) then
		(
		--	format "Not enougth particles in the specified system: gmValidNumParts: %,  gmParticleSystem.numParticlesGenerated(): %\n" gmValidNumParts (gmParticleSystem.numParticlesGenerated())
			lbContinue = querybox "Not enougth particles in the specified\nsystem to use the settings !!\nContinue Anyway ?" title:"Not Enought Particles"
			gmValidNumParts = gmRealNumParts
		)
		else if (PF_Source == (classof gmParticleSystem) and gmParticleSystem.numparticles() > gmValidNumParts) \
			or (PF_Source != (classof gmParticleSystem) and gmValidNumParts < gmRealNumParts) do
		--	or (particlepos gmParticleSystem (gmValidNumParts + 1)) != undefined do
			(
		--		format "Too much particles in the specified system: gmValidNumParts: %,  gmParticleSystem.numParticlesGenerated(): %\n" gmValidNumParts (gmParticleSystem.numParticlesGenerated())
				lbContinue = querybox "Too much particles in the specified\nsystem to use the settings !!\nContinue Anyway ?" title:"Too Much Particles"
			)

		if lbContinue and parent_chkbx.checked and undefined == gmParentObj do
		(
			messagebox "Please, Pick a Parent Object First" title:"No Parent Object"
			lbContinue = false
		)
		
		if lbContinue do
			with undo on
		--	with redraw off
			(
				FillParticleIndicesForKnots()
				CreateAndAnimateSplines()
			)
	)
)

------------------------------------------------------------------------------------------
--creation of the window, called WkPart2SplinWind

	unregisterRedrawViewsCallback drawPointCoordCallback
	forceCompleteRedraw()
	
	If WkPart2SplinWind != undefined do CloseRolloutFloater WkPart2SplinWind
	WkPart2SplinWind = NewrolloutFloater "Particles2Spline" 240 440 10 88
	
	Addrollout WkP2SRollAbout WkPart2SplinWind rolledup:true
	Addrollout WkP2SSettingsRoll WkPart2SplinWind
)
